#!/bin/bash

CONNECTION_MANAGER_CFG="/etc/wb-connection-manager.conf"

function update_priority() {
    local con_name="$1"
    local priority="$2"
    local configuration="$3"

    # Проверяем, существует ли файл конфигурации
    if [ ! -f "$configuration" ]; then
        echo "Configuration file '$configuration' does not exist. Creating it..."
        echo '{"connectivity_check_url": "http://network-test.debian.org/nm", "tiers": {"high": [], "low": [], "medium": []}}' > "$configuration"
    fi

    tmpfile=$(mktemp)

    # Удаляем соединение из всех приоритетов и записываем результат во временный файл
    jq --arg con_name "$con_name" '.tiers.high = [.tiers.high[] | select(. != $con_name)]
                                 | .tiers.medium = [.tiers.medium[] | select(. != $con_name)]
                                 | .tiers.low = [.tiers.low[] | select(. != $con_name)]' "$configuration" > "$tmpfile"

    case "$priority" in
        "high")
            jq --arg con_name "$con_name" '.tiers.high += [$con_name]' "$tmpfile" > "$tmpfile.new"
            ;;
        "medium")
            jq --arg con_name "$con_name" '.tiers.medium += [$con_name]' "$tmpfile" > "$tmpfile.new"
            ;;
        "low")
            jq --arg con_name "$con_name" '.tiers.low += [$con_name]' "$tmpfile" > "$tmpfile.new"
            ;;
    esac

    mv "$tmpfile.new" "$configuration"
    rm $tmpfile*
    cat "$configuration"
}

function upsert_wifi_connection() {
    local priority="$1"
    local con_name="$2"
    local ssid="$3"
    local password="$4"

    if [ -z "$priority" ] || [ -z "$con_name" ] || [ -z "$ssid" ] || [ -z "$password" ]; then
        echo "Usage: upsert_wifi_connection <priority> <con_name> <ssid> <password>"
        return 1
    fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type wifi con-name "$con_name" ifname wlan1 ssid "$ssid"
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    nmcli connection modify "$con_name" wifi-sec.key-mgmt wpa-psk
    nmcli connection modify "$con_name" wifi-sec.psk "$password"

    update_priority "$con_name" "$priority" "$CONNECTION_MANAGER_CFG"
}

function upsert_ethernet_connection() {
    local priority="$1"
    local con_name="$2"

    if [ -z "$priority" ] || [ -z "$con_name" ]; then
       echo "Usage: upsert_ethernet_connection <priority> <con_name>"
       return 1
   fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type ethernet con-name "$con_name" ifname eth0
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    update_priority "$con_name" "$priority" "$CONNECTION_MANAGER_CFG"
}

function upsert_gsm_connection() {
    local priority="$1"
    local con_name="$2"

    if [ -z "$priority" ] || [ -z "$con_name" ]; then
        echo "Usage: upsert_gsm_connection <priority> <con_name>"
        return 1
    fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type gsm con-name "$con_name" ifname "*"
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    # Дополнительные настройки для GSM-соединения
    nmcli connection modify "$con_name" gsm.auto-config true
    nmcli connection modify "$con_name" ipv4.method auto
    nmcli connection modify "$con_name" ipv6.addr-gen-mode default
    nmcli connection modify "$con_name" ipv6.method auto

    update_priority "$con_name" "$priority" "$CONNECTION_MANAGER_CFG"
}

function wait_for_network() {
    local timeout_in_seconds="$1"
    local node_address="$2"

    start_time=$(date +%s)

    while true; do
        if ping -c 1 "$node_address" &>/dev/null; then
            current_time=$(date "+%Y-%m-%d %H:%M:%S")
            echo "[${current_time}] Server is available! Continuing..."
            return 0
        else
            current_time=$(date "+%Y-%m-%d %H:%M:%S")
            elapsed_time=$(( $(date +%s) - start_time ))

            if [ $elapsed_time -ge $timeout_in_seconds ]; then
                echo "[${current_time}] Error! Network is not established within ${timeout_in_seconds} seconds. Exiting with an error."
                return 1
            fi

            echo "[${current_time}] Server is unavailable. Waiting..."
            sleep 1
        fi
    done
}
